local libraryObject, prevVersion = LibStub:NewLibrary("LibShorterSlash-1.0",1)
if not libraryObject then return end
if prevVersion then
	print("Prev:",prevVersion)
else
	print("Loaded!")
end

local ourCommands = {}

local recycleTable = {}
local function recursiveParse(prevCommands, inString, argsTable, handler)
	if (not inString) or (inString == '') then
		--print("'"..inString.."'")
		local ambigList
		for key in pairs(argsTable) do
			ambigList = (ambigList and ambigList.." " or "")..key
		end
		return "a", prevCommands.." ", (ambigList or "")
	end
	local nextParse, argString = inString:match("^(%S+)%s*(.*)")
	table.wipe(recycleTable)
	for key, data in pairs(argsTable) do
		local doesMatch = true
		for iNum=1, nextParse:len() do
			if nextParse:sub(iNum,iNum) ~= key:sub(iNum,iNum) then
				doesMatch = false
				break
			end
		end
		if doesMatch then
			tinsert(recycleTable,key)
		end
	end
	if #recycleTable >= 2 then
		local maxLen = 0
		for _, key in ipairs(recycleTable) do
			if key:len() > maxLen then
				maxLen = key:len()
			end
		end
		local matchingIndex = 0
		for strIndex=1,maxLen do
			local matchChar = recycleTable[1]:sub(strIndex,strIndex)
			for n=2,#recycleTable do
				if matchChar ~= recycleTable[n]:sub(strIndex,strIndex) then
					matchingIndex = strIndex-1
					break
				end
			end
			if matchingIndex > 0 then break end
		end
		return "a", prevCommands.." "..recycleTable[1]:sub(1,matchingIndex), table.concat(recycleTable," ")
	elseif #recycleTable == 1 then
		local matchedKey = recycleTable[1]
		local data = argsTable[matchedKey]
		if data.type == "group" then
			return recursiveParse(prevCommands.." "..matchedKey, argString, data.args, data.handler or handler)
		else
			return "c", data.handler or handler, prevCommands.." "..matchedKey, argString
		end
	end
end

local orig_ChatEdit_OnTabPressed = ChatEdit_OnTabPressed
ChatEdit_OnTabPressed = function(...)
	local editBox = ...
	local slashCommand,argString = editBox:GetText():match("/(%S+)%s*(.*)")
	--[[print("Command:",slashCommand)
	print("Arguments:",argString)--]]
	if slashCommand then
		local data = ourCommands[slashCommand]
		if data then
			local actionCode, actArg1, actArg2, actArg3 = recursiveParse("",argString,data)
			--[[print("Parses to:")
			print(actionCode)
			print(actArg1)
			print(actArg2)
			print(actArg3)--]]
			if actionCode == "a" then
				local newText = "/"..slashCommand..actArg1
				if editBox:GetText() == newText then
					print("Possible sub-commands:",actArg2)
				else
					editBox:SetText(newText)
				end
			elseif actionCode == "c" then
				editBox:SetText("/"..slashCommand..actArg2.." "..actArg3)
			else
				editBox:SetText("/"..slashCommand.." ")
			end
			editBox:SetCursorPosition(editBox:GetNumLetters())
			return
		end
	end
	orig_ChatEdit_OnTabPressed(...)
end

local function parseCommand(commandStr,input)
	local actionCode, actArg1, actArg2, actArg3 = recursiveParse("",input,ourCommands[commandStr])
	if actionCode == "a" then
		print("Ambigous command: /"..commandStr..actArg1)
		print("Possible sub-commands:",actArg2)
	end
	if actionCode == "c" then
		actArg1(actArg2:sub(2),actArg3)
	end
end

local function register(slashTab, ...)
	local tabIdent = select(1,...)
	if tabIdent:sub(1,1) == '/' then
		tabIdent = tabIdent:sub(2)
	end
	local upperIdent = tabIdent:upper()
	SlashCmdList[upperIdent] = function(input) parseCommand(tabIdent,input) end
	for i=1,select('#',...) do
		local cmd = select(i,...)
		if cmd:sub(1,1) == '/' then
			cmd = cmd:sub(2)
		end
		ourCommands[cmd] = slashTab
		_G["SLASH_"..upperIdent..i] = "/"..cmd
	end
end

function libraryObject:RegisterCommand(...)
	register(...)
end